@@ -1,8 +1,8 @@
 #  Phusion Passenger - https://www.phusionpassenger.com/
-#  Copyright (c) 2010-2018 Phusion Holding B.V.
+#  Copyright (c) 2010-2025 Asynchronous B.V.
 #
 #  "Passenger", "Phusion Passenger" and "Union Station" are registered
-#  trademarks of Phusion Holding B.V.
+#  trademarks of Asynchronous B.V.
 #
 #  Permission is hereby granted, free of charge, to any person obtaining a copy
 #  of this software and associated documentation files (the "Software"), to deal
@@ -142,8 +142,6 @@ module PhusionPassenger
     private_class_method :cc_or_cxx_supports_feliminate_unused_debug?
 
     def self.cc_or_cxx_supports_blocks?(language)
-      ext = detect_language_extension(language)
-      compiler_type_name = detect_compiler_type_name(language)
       command = create_compiler_command(language,"-E -dM",'- </dev/null')
       result = `#{command}`
       return result.include? "__BLOCKS__"
@@ -360,6 +358,13 @@ module PhusionPassenger
     end
     memoize :cc_supports_fno_limit_debug_info_flag?
 
+    def self.cc_supports_fno_optimize_sibling_calls_flag?
+      try_compile_with_warning_flag(
+        "Checking for C compiler '-fno-optimize-sibling-calls' support",
+        :c, '', '-fno-optimize-sibling-calls')
+    end
+    memoize :cc_supports_fno_optimize_sibling_calls_flag?
+
     def self.cxx_supports_visibility_flag?
       return false if os_name_simple == "aix"
       return try_compile("Checking for C++ compiler '-fvisibility' support",
@@ -430,6 +435,13 @@ module PhusionPassenger
     end
     memoize :cxx_supports_fno_limit_debug_info_flag?
 
+    def self.cxx_supports_fno_optimize_sibling_calls_flag?
+      try_compile_with_warning_flag(
+        "Checking for C++ compiler '-fno-optimize-sibling-calls' support",
+        :cxx, '', '-fno-optimize-sibling-calls')
+    end
+    memoize :cxx_supports_fno_optimize_sibling_calls_flag?
+
     def self.cc_supports_no_tls_direct_seg_refs_option?
       return try_compile("Checking for C compiler '-mno-tls-direct-seg-refs' support",
         :c, '', '-mno-tls-direct-seg-refs')
@@ -469,6 +481,13 @@ module PhusionPassenger
     end
     memoize :cxx_block_support_ok?, true
 
+    def self.cxx_supports_wno_vla_cxx_extension_flag?
+      return try_compile_with_warning_flag(
+        "Checking for C++ compiler '-Wno-vla-cxx-extension' support",
+        :cxx, '', '-Wno-vla-cxx-extension')
+    end
+    memoize :cxx_supports_wno_vla_cxx_extension_flag?, true
+
     # Returns whether compiling C++ with -fvisibility=hidden might result
     # in tons of useless warnings, like this:
     # http://code.google.com/p/phusion-passenger/issues/detail?id=526
@@ -493,10 +512,10 @@ module PhusionPassenger
     end
     memoize :cxx_visibility_flag_generates_warnings?, true
 
-    def self.address_sanitizer_flag
+    def self.address_sanitizer_flags
       if cc_is_clang?
         if `#{cc} --help` =~ /-fsanitize=/
-          "-fsanitize=address"
+          "-fsanitize=address -fsanitize-address-use-after-return=always"
         else
           "-faddress-sanitizer"
         end
@@ -504,27 +523,30 @@ module PhusionPassenger
         nil
       end
     end
-    memoize :address_sanitizer_flag
 
-    def self.cxx_11_flag
-      # C++11 support on FreeBSD 10.0 + Clang seems to be bugged.
-      # http://llvm.org/bugs/show_bug.cgi?id=18310
-      return nil if os_name_simple == "freebsd"
+    def self.undefined_behavior_sanitizer_flags
+      if cc_is_clang?
+        "-fsanitize=undefined"
+      else
+        nil
+      end
+    end
 
+    def self.cxx_14_flag
       source = %{
         struct Foo {
           Foo(Foo &&f) { }
         };
       }
-      if try_compile("Checking for C++ -std=gnu++11 compiler flag", :cxx, source, '-std=gnu++11')
-        return "-std=gnu++11"
-      elsif try_compile("Checking for C++ -std=c++11 compiler flag", :cxx, source, '-std=c++11')
-        return "-std=c++11"
+      if try_compile("Checking for C++ -std=gnu++14 compiler flag", :cxx, source, '-std=gnu++14')
+        return "-std=gnu++14"
+      elsif try_compile("Checking for C++ -std=c++14 compiler flag", :cxx, source, '-std=c++14')
+        return "-std=c++14"
       else
         return nil
       end
     end
-    memoize :cxx_11_flag, true
+    memoize :cxx_14_flag, true
 
     def self.has_rt_library?
       return try_link("Checking for -lrt support",
@@ -544,12 +566,6 @@ module PhusionPassenger
     end
     memoize :has_dl_library?, true
 
-    def self.has_alloca_h?
-      return try_compile("Checking for alloca.h",
-        :c, '#include <alloca.h>')
-    end
-    memoize :has_alloca_h?, true
-
     def self.has_accept4?
       return try_compile("Checking for accept4()", :c, %Q{
         #define _GNU_SOURCE
@@ -584,7 +600,7 @@ module PhusionPassenger
       # -ggdb instead.
       #
       # In any case we'll always want to use -ggdb for better GDB debugging.
-      if cc_is_gcc?
+      if cxx_is_gcc?
         result = '-ggdb'
       else
         result = '-g'
@@ -603,6 +619,14 @@ module PhusionPassenger
       end
     end
 
+    def self.precompiled_header_extension
+      if cxx_is_gcc?
+        'gch'
+      else
+        'pch'
+      end
+    end
+
 
     def self.make
       return string_env('MAKE', find_command('make'))
